/*****************************************************************************************************
*Critter Camera Sketch
*Joel Bartlett
*Sparkfun Electronics 
*November 22, 2012
*This code is used to control a CMOS camera, connected to two servos, via a bluetoth connection.
*The servos are glued to each other so as to move the camera in a pan/tilt fassion
*There is also a SHT15 temerature and humidity sensor attached to the camera.
*All of the commands are issued with the keyboard and transmitted over bluetooth to the unit attached to the terrarium. 
*This code borrows haevily from the SHT15 Wiring example found here: http://wiring.org.co/learning/basics/humiditytemperaturesht15.html
*******************************************************************************************************/

#include <string.h>
#include <Servo.h>

Servo Pan; //declare the pan servo instance using the servo library
Servo Tilt; // declare the tilt servo instance

int panDeg; //varaible to store degrees for pan
int tiltDeg; //varaible to store degrees for tilt

int led1 = 6; //varables for LEDs, LEDs were not used in the final model
int led2 = 7;

int temperatureCommand  = B00000011;  // command used to read temperature 
int humidityCommand = B00000101;  // command used to read humidity 
 
int clockPin = 2;  // pin used for clock 
int dataPin  = 3;  // pin used for data 
int ack;  // track acknowledgment for errors 

int val; //variables for converting Celcius to fahrenheit
float tempF;          
float tempC;
float humidity; 
//-------------------------------------------------------------------
void setup ()
{
  Serial.begin(9600); //set Serial baud rate to 9600. The Bluetooth also runs at this baud rate 

  panDeg  = 90; //Set varaibles to 90 degrees
  tiltDeg = 90;

  Pan.attach(9); // attach the pan servo to digital pin 9 (PWM)
  Tilt.attach(10); // attach the pan servo to digital pin 10 (PWM)
  Pan.write(panDeg); //initialize the servos with the set variables (centered)
  Tilt.write(tiltDeg);
  
  //This prints out the menu in a somewhat formatted fashion 
  Serial.print("\t");
  Serial.println("\tCamera Controls");
  Serial.println("_______________________________________");
  Serial.println("\t\tw = up x1");
  Serial.print("a = left x1");
  Serial.print("\ts = down x1");
  Serial.println("\td = right x1");  
  
  Serial.println("\n\t\ti = up x5");
  Serial.print("j = left x5");
  Serial.print("\tk = down x5");
  Serial.println("\tl = right x5");
  
  Serial.println("\n\t\t   8 = up preset");
  Serial.print("4 = left preset");
  Serial.print("\t   5 = middle preset");
  Serial.println("\t6 = right preset");
  Serial.println("\t\t   2 = down preset");
  
  Serial.println("\n\n\tT = Display Temperature and Humidity");
  Serial.println("\tY = Dispaly current Pan/Tilt coordinates");
  Serial.println("\tR = RESET");

  pinMode(led1, OUTPUT); // not used on final model
  pinMode(led2, OUTPUT);
  digitalWrite(led1, LOW);
  digitalWrite(led2, LOW);
}
//-----------------------------------------------------------------
void loop ()
{

char temp; // create variable to read incoming serial data

 if(Serial.available())// if there is data on the serial line....
 {
  temp = Serial.read(); //store the incoming byte to our temporay variable
  
  if(temp == 119 || temp == 87)//dec for 'w'
  {
    ++tiltDeg; //move servo up by 1 degree
  }
    if(temp == 115 || temp == 83)//dec for 's'
  {
    --tiltDeg; //move servo down by 1 degree
  }
  if(temp == 97 || temp == 65)//dec for 'a'
  {
    ++panDeg; //turn servo left by 1 degree
  }
    if(temp == 100 || temp == 68)//dec for 'd'
  {
    --panDeg; //turn servo right by 1 degree
  }
  
    if(temp == 105 || temp == 73)//dec for 'i'
  {
    tiltDeg += 5; //increase servo up by 5 degrees
  }
    if(temp == 107 || temp == 75)//dec for 'k'
  {
    tiltDeg -= 5;
  }
  if(temp == 106 || temp == 74)//dec for 'j'
  {
    panDeg += 5;
  }
    if(temp == 108 || temp == 76)//dec for 'l'
  {
    panDeg -= 5;
  }
  
  //These correspond to the preset positions 
  if(temp == '5') 
  {
    panDeg = 90;
    tiltDeg = 90;
  }
    if(temp == '2') 
  {
     panDeg = 90;
    tiltDeg = 45;
  }
    if(temp == '6') 
  {
    panDeg = 0;
    tiltDeg = 90;
  }
    if(temp == '4') 
  {
   panDeg = 180;
    tiltDeg = 90;
  }
      if(temp == '8')
  {
    panDeg = 90;
    tiltDeg = 135;
  }
  
  if(temp == 116 || temp == 84)// dec for T
  sht15(); //prints out the temp and humidity 
  
  if(temp == 82 || temp == 114)//dec for R RESET
  {
  digitalWrite(led2, HIGH);
  delay(20);
  digitalWrite(led2, LOW); 
  setup();// resets the system, servo goes back to center
  }
  
  if(temp == 'y' || temp =='Y')
  {
    //print the current position of the servos
    Serial.print("Pan = ");
    Serial.print(panDeg);
    Serial.print(", Tilt = ");
    Serial.println(tiltDeg);
    digitalWrite(led1, HIGH);
    delay(20);
    digitalWrite(led1, LOW); 

  }
  
 Pan.write(panDeg); //take current values based on input and write to the servos
 Tilt.write(tiltDeg);
 }
}
//------------------------------------------------------------------------
void sht15()
{
  // read the temperature and convert it to centigrades 
  sendCommandSHT(temperatureCommand, dataPin, clockPin); 
  waitForResultSHT(dataPin); 
  val = getData16SHT(dataPin, clockPin); 
  skipCrcSHT(dataPin, clockPin); 
  tempC = (float)val * 0.01 - 40; //convert C to F
  tempF = (float)tempC * 1.8 + 32;
  Serial.print("Temperature: "); 
  Serial.print((long)tempF, DEC);
  Serial.print("_"); 
  Serial.print((char)176); 
  Serial.print("F "); 
  // read the humidity 
  sendCommandSHT(humidityCommand, dataPin, clockPin); 
  waitForResultSHT(dataPin); 
  val = getData16SHT(dataPin, clockPin); 
  skipCrcSHT(dataPin, clockPin); 
  humidity = -4.0 + 0.0405 * val + -0.0000028 * val * val; 
  Serial.print(" Relative Humidity: "); 
  Serial.print((long)humidity, DEC); 
   Serial.println("%");
  //delay(1000); // wait for 3 sec for next reading 
}
 
 
// commands for reading/sending data to a SHTx sensor 
 
int shiftIn(int dataPin, int clockPin, int numBits) { 
  int ret = 0; 
 
  for (int i=0; i<numBits; ++i) { 
    digitalWrite(clockPin, HIGH); 
    //delay(10); not needed :) 
    ret = ret*2 + digitalRead(dataPin); 
    digitalWrite(clockPin, LOW); 
  } 
  return(ret); 
} 
 
// send a command to the SHTx sensor 
void sendCommandSHT(int command, int dataPin, int clockPin) { 
  int ack; 
 
  // transmission start 
  pinMode(dataPin, OUTPUT); 
  pinMode(clockPin, OUTPUT); 
  digitalWrite(dataPin, HIGH); 
  digitalWrite(clockPin, HIGH); 
  digitalWrite(dataPin, LOW); 
  digitalWrite(clockPin, LOW); 
  digitalWrite(clockPin, HIGH); 
  digitalWrite(dataPin, HIGH); 
  digitalWrite(clockPin, LOW); 
  
  // shift out the command (the 3 MSB are address and must be 000, the last 5 bits are the command) 
  shiftOut(dataPin, clockPin, MSBFIRST, command); 
  
  // verify we get the right ACK 
  digitalWrite(clockPin, HIGH); 
  pinMode(dataPin, INPUT); 
  ack = digitalRead(dataPin); 
  if (ack != LOW) 
    Serial.println("ACK error 0"); 
  digitalWrite(clockPin, LOW); 
  ack = digitalRead(dataPin); 
  if (ack != HIGH) 
    Serial.println("ACK error 1"); 
} 
 
// wait for the SHTx answer 
void waitForResultSHT(int dataPin) { 
  int ack; 
 
  pinMode(dataPin, INPUT); 
  for(int i=0; i<100; ++i) { 
    delay(10); 
    ack = digitalRead(dataPin); 
    if (ack == LOW) 
      break; 
  } 
  if (ack == HIGH) 
    Serial.println("ACK error 2"); 
} 
 
// get data from the SHTx sensor 
int getData16SHT(int dataPin, int clockPin) { 
  int val; 
 
  // get the MSB (most significant bits) 
  pinMode(dataPin, INPUT); 
  pinMode(clockPin, OUTPUT); 
  val = shiftIn(dataPin, clockPin, 8); 
  val *= 256; // this is equivalent to val << 8; 
  
  // send the required ACK 
  pinMode(dataPin, OUTPUT); 
  digitalWrite(dataPin, HIGH); 
  digitalWrite(dataPin, LOW); 
  digitalWrite(clockPin, HIGH); 
  digitalWrite(clockPin, LOW); 
  
  // get the LSB (less significant bits) 
  pinMode(dataPin, INPUT); 
  val |= shiftIn(dataPin, clockPin, 8); 
  return val; 
} 
 
// skip CRC data from the SHTx sensor 
void skipCrcSHT(int dataPin, int clockPin) { 
  pinMode(dataPin, OUTPUT); 
  pinMode(clockPin, OUTPUT); 
  digitalWrite(dataPin, HIGH); 
  digitalWrite(clockPin, HIGH); 
  digitalWrite(clockPin, LOW); 
} 
